import QtQuick 1.0
import "components"
import "scripts/createobject.js" as ObjectCreator

Page {

    title: qsTr("Accounts")

    function showYouTubeAuthDialog() {
        var authDialog = ObjectCreator.createObject(Qt.resolvedUrl("YTAuthDialog.qml"), appWindow);
        authDialog.open();
    }

    function showFacebookAuthDialog() {
        var authDialog = ObjectCreator.createObject(Qt.resolvedUrl("FacebookAuthDialog.qml"), appWindow);
        authDialog.open();
    }

    function showTwitterAuthDialog() {
        var authDialog = ObjectCreator.createObject(Qt.resolvedUrl("TwitterAuthDialog.qml"), appWindow);
        authDialog.open();
    }

    Connections {
        target: Dailymotion
        onUserSignedInChanged: if (Dailymotion.userSignedIn) messages.displayMessage(qsTr("You are signed in to your Dailymotion account"));
    }

    Connections {
        target: YouTube
        onUserSignedInChanged: if (YouTube.userSignedIn) messages.displayMessage(qsTr("You are signed in to your YouTube account"));
    }

    Connections {
        target: Sharing
        onUserSignedInToFacebookChanged: {
            if (Sharing.userSignedInToFacebook) {
                messages.displayMessage(qsTr("You are signed in to your facebook account"));
            }
            else {
                messages.displayMessage(qsTr("Your facebook credentials have been deleted, please go to the website to revoke access"));
            }
        }
        onUserSignedInToTwitterChanged: {
            if (Sharing.userSignedInToTwitter) {
                messages.displayMessage(qsTr("You are signed in to your twitter account"));
            }
            else {
                messages.displayMessage(qsTr("Your twitter credentials have been deleted, please go to the website to revoke access"));
            }
        }
    }

    Flickable {
        id: flicker

        anchors { fill: parent; topMargin: titleArea.height }
        contentWidth: width
        contentHeight: col1.height + 20
        flickableDirection: Flickable.VerticalFlick
        clip: true

        Column {
            id: col1

            z: 101
            anchors { top: parent.top; left: parent.left; right: parent.right; margins: 10 }
            spacing: 20

            SeparatorLabel {
                width: parent.width
                text: qsTr("YouTube")
            }

            Label {
                width: parent.width
                font.bold: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap

                text: YouTube.userSignedIn ? qsTr("You are signed in to your YouTube account") : qsTr("Sign in to your YouTube account")
            }

            Button {
                x: Math.floor((parent.width / 2) - (width  / 2))
                text: YouTube.userSignedIn ? qsTr("Sign out") : qsTr("Sign in")
                onClicked: YouTube.userSignedIn ? YouTube.signOut() : showYouTubeAuthDialog()
            }

            Label {
                width: parent.width
                color: Settings.activeColor
                font.bold: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                opacity: ytMouseArea.pressed ? 0.5 : 1
                text: qsTr("No YouTube account? Click here to sign up.")
                visible: !YouTube.userSignedIn

                MouseArea {
                    id: ytMouseArea

                    anchors.fill: parent
                    onClicked: Qt.openUrlExternally("http://youtube.com/signup")
                }
            }

            SeparatorLabel {
                width: parent.width
                text: qsTr("Dailymotion")
            }

            Label {
                width: parent.width
                font.bold: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                text: Dailymotion.userSignedIn ? qsTr("You are signed in to your Dailymotion account") : qsTr("Sign in to your Dailymotion account")
            }

            TextField {
                id: userInput

                width: parent.width
                placeholderText: qsTr("Username")
                visible: !Dailymotion.userSignedIn
            }

            TextField {
                id: passwordInput

                width: parent.width
                echoMode: TextInput.Password
                placeholderText: qsTr("Password")
                visible: !Dailymotion.userSignedIn
            }

            Label {
                width: parent.width
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                text: qsTr("Note: Your username and password are not stored by cuteTube")
                visible: !Dailymotion.userSignedIn
            }

            Button {
                x: Math.floor((parent.width / 2) - (width  / 2))
                text: Dailymotion.userSignedIn ? qsTr("Sign out") : qsTr("Sign in")
                enabled: (Dailymotion.userSignedIn) || ((userInput.text !== "") && (passwordInput.text !== ""))
                onClicked: Dailymotion.userSignedIn ? Dailymotion.signOut() : Dailymotion.signIn(userInput.text, passwordInput.text)
            }

            Label {
                width: parent.width
                color: Settings.activeColor
                font.bold: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                opacity: scMouseArea.pressed ? 0.5 : 1
                text: qsTr("No Dailymotion account? Click here to sign up.")
                visible: !Dailymotion.userSignedIn

                MouseArea {
                    id: scMouseArea

                    anchors.fill: parent
                    onClicked: Qt.openUrlExternally("http://dailymotion.com/register")
                }
            }

            SeparatorLabel {
                width: parent.width
                text: qsTr("Sharing")
            }

            Label {
                width: parent.width
                font.bold: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap

                text: Sharing.userSignedInToFacebook ? qsTr("You are signed in to your facebook account") : qsTr("Sign in to your facebook account")
            }

            Button {
                x: Math.floor((parent.width / 2) - (width  / 2))
                text: Sharing.userSignedInToFacebook ? qsTr("Sign out") : qsTr("Sign in")
                onClicked: Sharing.userSignedInToFacebook ? Settings.clearFacebookAccount() : showFacebookAuthDialog()
            }

            Label {
                width: parent.width
                color: Settings.activeColor
                font.bold: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                opacity: fbMouseArea.pressed ? 0.5 : 1
                text: qsTr("No facebook account? Click here to sign up.")
                visible: !Sharing.userSignedInToFacebook

                MouseArea {
                    id: fbMouseArea

                    anchors.fill: parent
                    onClicked: Qt.openUrlExternally("http://www.facebook.com/r.php")
                }
            }

            Label {
                width: parent.width
                font.bold: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap

                text: Sharing.userSignedInToTwitter ? qsTr("You are signed in to your twitter account") : qsTr("Sign in to your twitter account")
            }

            Button {
                x: Math.floor((parent.width / 2) - (width  / 2))
                text: Sharing.userSignedInToTwitter ? qsTr("Sign out") : qsTr("Sign in")
                onClicked: Sharing.userSignedInToTwitter ? Settings.clearTwitterAccount() : showTwitterAuthDialog()
            }

            Label {
                width: parent.width
                color: Settings.activeColor
                font.bold: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                opacity: twMouseArea.pressed ? 0.5 : 1
                text: qsTr("No twitter account? Click here to sign up.")
                visible: !Sharing.userSignedInToTwitter

                MouseArea {
                    id: twMouseArea

                    anchors.fill: parent
                    onClicked: Qt.openUrlExternally("https://twitter.com/")
                }
            }
        }
    }
}
